"""
Python translation of fit..
which is copyright (c) 2002 Cunningham & Cunningham, Inc.
Released under the terms of the GNU General Public License version 2 or later.
"""

import string, re
from types import *
from rexec import RExec

## Factory ##################################

def adapterOnType(fixture,atype):
    """
    Return a dummy adapter which can parse a string to this type
    """
    a = adapterFor(atype)
    a.init(fixture, atype)
    return a

def adapterOnField(fixture, fieldname, targetclass=None):
    """
    Return an adapter adapting this field of fixture's class (or
    targetclass) to the fit TypeAdapter interface (in the context of
    fixture).
    """
    if not targetclass: targetclass = fixture.__class__
    atype = type(getattr(targetclass,fieldname))
    a = adapterFor(atype)
    a.init(fixture, atype)
    a.target = fixture
    a.field = fieldname
    return a
        
def adapterOnMethod(fixture, method):
    """
    Return an adapter adapting this unbound method to the fit
    TypeAdapter interface (in the context of fixture).
    """
    a = adapterFor(UnboundMethodType)
    a.init(fixture, UnboundMethodType)
    a.target = fixture
    a.method = method
    return a
        
##public static TypeAdapter adapterFor(Class type) throws UnsupportedOperationException {
def adapterFor(atype):
    return AutoAdapter() # ha!
    if   atype == StringType: return StringAdapter()
    elif atype == IntType: return IntAdapter()
    elif atype == LongType: return LongAdapter()
    elif atype == FloatType: return FloatAdapter()
    elif atype == ComplexType: return ComplexAdapter()
    #elif atype == ListType: return ListAdapter()
    #elif atype == UnboundMethodType: return StringAdapter()
    #else: return TypeAdapter()
    elif atype == UnboundMethodType: return AutoAdapter()
    else: return AutoAdapter()

## base class ####################################

class TypeAdapter:
    """
    """
    fixture = None
    type    = None
    target  = None
    field   = None
    method  = None

    ##void init (Fixture fixture, Class type) {
    def init(self,fixture,atype):
        self.fixture = fixture
        self.type = atype

    ##public Object get() throws IllegalAccessException, InvocationTargetException {
    def get(self):
        if self.field: return getattr(self.target,self.field)
        elif self.method: return self.invoke()
        else: return None

    ##public void set(Object value) throws IllegalAccessException {
    def set(self,value):
        if self.field:
            setattr(self.target,self.field,value)

    ##public Object invoke() throws IllegalAccessException, InvocationTargetException {
    def invoke(self):
        return self.method(self.target)
    
    ##public Object parse(String s) throws Exception {
    def parse(self,s):
        return self.fixture.parse(s, self.type)
    
    ##public boolean equals(Object a, Object b) {
    def equals(self,a,b):
        # value matching
        #import sys; sys.stderr.write('comparing '+repr(a)+' \t'+repr(b))
        return a == b
        # string literal matching
        #import sys; sys.stderr.write('comparing '+repr(repr(a))+' \t'+repr(repr(b)))
        #return repr(a) == repr(b)

    ##public String toString(Object o) {
    def toString(self,o):
        #return repr(o)
        return re.sub(r"^'(.+)'$",r"\1",repr(o))

## Subclasses ##############################
# these are forgiving parsers, which may or may not be what we want

class StringAdapter(TypeAdapter):
    def parse(self,s):
        return s.strip()

class IntAdapter(TypeAdapter):
    def parse(self,s):
        return int(s)

class LongAdapter(TypeAdapter):
    def parse(self,s):
        return long(s)

class FloatAdapter(TypeAdapter):
    def parse(self,s):
        return float(s)

class ComplexAdapter(TypeAdapter):
    def parse(self,s):
        return complex(s)

#class ListAdapter(TypeAdapter):
#    pass
#    componentType = None
#    componentAdapter = None
#
#    ##void init(Fixture target, Class type) {
#    def init(self,target,type)
#        TypeAdapter.init(self,target,type)
#        self.componentType = StringType #type.getComponentType()
#        self.componentAdapter = adapterOnType(target, self.componentType)
#
#    ##public Object parse(String s) throws Exception {
#    def parse(self,s):
#        StringTokenizer t = StringTokenizer(s, ",")
#        Object array = Array.newInstance(componentType, t.countTokens())
#        for (int i=0; t.hasMoreTokens(); i++)):
#            Array.set(array, i, componentAdapter.parse(t.nextToken()))
#        return array
#
#    ##public String toString(Object o) {
#        int length = Array.getLength(o)
#        StringBuffer b = StringBuffer(5*length)
#        for (int i=0; i<length; i++)):
#            b.append(componentAdapter.toString(Array.get(o, i)))
#            if (i < (length-1))):
#                b.append(',')
#        return b.toString()
#
#    ##public boolean equals(Object a, Object b) {
#        int length = Array.getLength(a)
#        if (length != Array.getLength(b)) return false
#        for (int i=0; i<length; i++)):
#            if (!componentAdapter.equals(Array.get(a,i), Array.get(b,i))) return false
#        return true

class AutoAdapter(TypeAdapter):
    """
    This adapter returns a type based on the format of the table cell
    contents, following python's rules for literals (plus a few others).
    We could fall back on this when we don't know what type is expected.
    I am lazy and so use rexec to do the work.
    
    #Note: The RExec class can prevent code from performing unsafe
    #operations like reading or writing disk files, or using TCP/IP
    #sockets. However, it does not protect against code using
    #extremely large amounts of memory or processor time.
    """
    r_eval = RExec().r_eval
    def parse(self,s):
        if s.strip().lower() == 'true':
            return 1
        elif s.strip().lower() == 'false':
            return 0
        else:
            try:
                return self.r_eval(s)
            except SystemExit:
                return None
            except:
                return s
